// Extension Preferences Menu
// Author: Antonius32
// Version 2.13.7 (February 2016)

var ExtensionPreferencesMenu = {
	startup: function() {
		window.removeEventListener('load', function EPMload() { ExtensionPreferencesMenu.startup() });

		Components.utils.import("resource://gre/modules/AddonManager.jsm");
		AddonManager.addAddonListener(this);

		Components.utils.import("resource://gre/modules/Services.jsm");
		this._branch = Services.prefs.getBranch("extensions.ExtensionPreferencesMenu.");
		this._branch.addObserver("", this, false);

		window.addEventListener('customizationchange', function EPMcustomizationchange() { ExtensionPreferencesMenu.setup() });

		if (this._branch.getBoolPref("FirstRun"))
			this.onFirstRun();

		this.setup();
	},

	observe: function(aSubject, aTopic, aData) {
		if (aTopic == "nsPref:changed")
			this.setup();
	},

	onFirstRun: function() {
		this._branch.setBoolPref("FirstRun", false);

		// Pale Moon
		let NavBar = document.getElementById("nav-bar");
		if (NavBar) {
			// Add the toolbar button to the Navigation toolbar
			let DownloadsButton = document.getElementById("downloads-button");
			NavBar.contains(DownloadsButton) ? NavBar.insertItem("extensionPrefsMenu-button", DownloadsButton.nextSibling) : NavBar.insertItem("extensionPrefsMenu-button");
			NavBar.setAttribute("currentset", NavBar.currentSet);
			document.persist("nav-bar", "currentset");

			// Make sure the Navigation toolbar is visible
			if (NavBar.collapsed) {
				NavBar.collapsed = false;
				document.persist("nav-bar", "collapsed");
			}
		}

		// FossaMail
		let MailBar = document.getElementById("mail-bar3");
		if (MailBar) {
			// Add the toolbar button to the Mail toolbar
			let GlobalSearchBox = document.getElementById("gloda-search");
			MailBar.contains(GlobalSearchBox) ? MailBar.insertItem("extensionPrefsMenu-button", GlobalSearchBox.nextSibling) : MailBar.insertItem("extensionPrefsMenu-button");
			MailBar.setAttribute("currentset", MailBar.currentSet);
			document.persist("mail-bar3", "currentset");

			// Make sure the Mail toolbar is visible
			if (MailBar.collapsed) {
				MailBar.collapsed = false;
				document.persist("mail-bar3", "collapsed");
			}
		}
	},

	setup: function() {
		let EPMappMenuItem = document.getElementById("appmenu_extensionPrefs");
		EPMappMenuItem.collapsed = !this._branch.getBoolPref("InAppMenu");
		let OrigAppMenuItem = document.getElementById("appmenu_addons");
		OrigAppMenuItem.collapsed = !EPMappMenuItem.collapsed;

		this.populate(EPMappMenuItem.firstChild);

		let EPMtoolsMenuItem = document.getElementById("extensionPrefsMenu");
		EPMtoolsMenuItem.collapsed = !this._branch.getBoolPref("InToolsMenu");
		let OrigToolsMenuItem = document.getElementById("menu_openAddons") || document.getElementById("addonsManager");
		OrigToolsMenuItem.collapsed = !EPMtoolsMenuItem.collapsed;

		this._branch.getBoolPref("IconInToolsMenu") ? EPMtoolsMenuItem.setAttribute("iconic", "true") : EPMtoolsMenuItem.removeAttribute("iconic");

		this.populate(EPMtoolsMenuItem.firstChild);

		let EPMtoolbarButton = document.getElementById("extensionPrefsMenu-button");
		if (EPMtoolbarButton)
			this.populate(EPMtoolbarButton.firstChild);
	},

	populate: function(aParent) {
		let item, MenuIconURL, EPMShowIt = true, EPMblackListArray = [];
		let EPMshowVersion = this._branch.getBoolPref("ShowVersion");
		let EPMshowIcons = this._branch.getBoolPref("ShowIcons");
		let EPMshowAll = this._branch.getBoolPref("ShowAll");
		let EPMshowDisabled = this._branch.getBoolPref("ShowDisabled");
		let EPMshowIncompatible = this._branch.getBoolPref("ShowIncompatible");
		let EPMquickActions = this._branch.getBoolPref("QuickActions");
		let EPMshowRestart = this._branch.getBoolPref("ShowRestart");
		let EPMrestartLabel = document.getElementById("EPMstring-bundle").getString("EPMRestart");
		let EPMopenApp = window['BrowserOpenAddonsMgr'] || window['openAddonsMgr']; 
		let EPMscrollInToView = this._branch.getBoolPref("ScrollInToView");
		let EPMhideSigned = this._branch.getBoolPref("HideSigned");
		let EPMblackList = this._branch.getCharPref("BlackList");
		if (EPMblackList) {
			EPMblackList = EPMblackList.split("").reverse().join("");
			EPMblackListArray = EPMblackList.split(/\,(?!\\)/);
			for (let i = 0; i < EPMblackListArray.length; i++)
				EPMblackListArray[i] = EPMblackListArray[i].split("").reverse().join("").replace(/^ | $/g,"").replace(/\\\,/g,",").toLowerCase();
		}

		AddonManager.getAllAddons(function(addon) {
			while (aParent.childNodes.length)
				aParent.removeChild(aParent.firstChild);

			addon.forEach(function(EPMaddon) {
				if (EPMaddon.type == "extension" && (!EPMaddon.appDisabled || (EPMaddon.appDisabled && EPMshowIncompatible)) && ((EPMaddon.isActive && EPMaddon.optionsURL) || ((EPMaddon.userDisabled && EPMshowDisabled) || (!EPMaddon.userDisabled && EPMshowAll) || (EPMaddon.appDisabled && EPMshowIncompatible)))) {
					item = document.createElement("menuitem");
					item.setAttribute("label", EPMshowVersion ? EPMaddon.name + "  " + (EPMhideSigned ? EPMaddon.version.replace(".1-signed", ".1") : EPMaddon.version) : EPMaddon.name);
					if (EPMaddon.description)
						item.setAttribute("tooltiptext", EPMaddon.description);
					item.addEventListener("command", function(event) {
						if (EPMaddon.optionsURL && !event.ctrlKey) {
							switch (Number(EPMaddon.optionsType)) {
								case 2:
									EPMopenApp('addons://detail/' + encodeURIComponent(EPMaddon.id) + (EPMscrollInToView ? '/preferences' : ''));
									break;
								case 3:
									switchToTabHavingURI(EPMaddon.optionsURL, true);
									break;
								default:
									openDialog(EPMaddon.optionsURL, EPMaddon.name, 'chrome,titlebar,toolbar,resizable,scrollbars,centerscreen,dialog=no,modal=no');
								}
						}
					});
					item.addEventListener("click", function(event) {
						if (EPMquickActions) {
							if (event.button == 1 && EPMaddon.homepageURL) {
								if ("gBrowser" in window) { // Pale Moon
									switchToTabHavingURI(EPMaddon.homepageURL, true);
									let AppMenuPopup = document.getElementById('appmenu-popup');
									let ToolsPopup = document.getElementById('menu_ToolsPopup');
									let EPMbuttonPopup = document.getElementById('EPMbuttonPopup');
									if (AppMenuPopup.state == 'open')
										AppMenuPopup.hidePopup();
									else if (ToolsPopup.state == 'open')
										ToolsPopup.hidePopup();
									else
										EPMbuttonPopup.hidePopup();
								}
								else // FossaMail
									Cc["@mozilla.org/messenger;1"].createInstance(Ci.nsIMessenger).launchExternalURL(EPMaddon.homepageURL);
							}
							if (event.button == 2 || (event.button == 0 && event.ctrlKey))
								AddonManager.getAddonByID(EPMaddon.id, function(addon) {
									addon.userDisabled = !addon.userDisabled;
									if (addon.operationsRequiringRestart && !EPMshowRestart)
										Application.restart();
									ExtensionPreferencesMenu.setup();
								});
						}
					});
					if (!EPMaddon.optionsURL)
						item.setAttribute("disabled", true);
					if (!EPMaddon.isActive)
						item.setAttribute("EPMisDisabled", true);
					if (EPMaddon.pendingOperations == 1)
						item.setAttribute("EPMtoBeToggled", true);
					if (EPMaddon.pendingOperations == 2)
						item.setAttribute("EPMtoBeToggled", false);

					if (EPMshowIcons) {
						item.setAttribute("class", "menuitem-iconic");
						MenuIconURL = EPMaddon.iconURL || 'chrome://mozapps/skin/extensions/extensionGeneric-16.png';
						item.setAttribute("style", "list-style-image: url(" + MenuIconURL + ")");
					}
					else
						item.setAttribute("class", "menuitem");

					if (EPMblackListArray)
						for (let i = 0; i < EPMblackListArray.length; i++)
							if (EPMblackListArray[i] == EPMaddon.name.toLowerCase()) {
								EPMShowIt = false;
								break;
							}
							else
								EPMShowIt = true;

					if (EPMShowIt)
						aParent.appendChild(item);
				}
			});

			Cc["@mozilla.org/xul/xul-sort-service;1"].getService(Ci.nsIXULSortService).sort(aParent, "label", "ascending");

			if (EPMshowRestart) {
				aParent.appendChild(document.createElement("menuseparator"));

				item = document.createElement("menuitem");
				item.setAttribute("label", EPMrestartLabel);
				item.addEventListener("command", function() { Application.restart() });

				if (EPMshowIcons) {
					item.setAttribute("class", "menuitem-iconic");
					item.setAttribute("style", "list-style-image: url(chrome://prefsmenu/skin/restart.png)");
				}
				else
					item.setAttribute("class", "menuitem");

				aParent.appendChild(item);
			}
		});
	},

	openAddonsManager: function() {
		(window['BrowserOpenAddonsMgr'] || window['openAddonsMgr'])((this._branch.getBoolPref("OpenExtensionsPage")) ? 'addons://list/extension' : '');
	},

	shutdown: function() {
		window.removeEventListener('unload', function EPMunload() { ExtensionPreferencesMenu.shutdown() });
		AddonManager.removeAddonListener(this);
		this._branch.removeObserver("", this);
		window.removeEventListener('customizationchange', function EPMcustomizationchange() { ExtensionPreferencesMenu.setup() });
	}
};

window.addEventListener('load', function EPMload() { ExtensionPreferencesMenu.startup() });
window.addEventListener('unload', function EPMunload() { ExtensionPreferencesMenu.shutdown() });
